<?php
namespace Util;

class UtilsIntegracao {
    protected $status;
    protected $mensagens;
    protected $is_dev_mode;
    public function __construct($is_dev_mode){
        $this->status = 'S';
        $this->mensagens = array();
        $this->is_dev_mode = $is_dev_mode;
        //Auxiliar para habilitar modo de desenvolvimento a partir da presença do header DEV
        if(isset($_SERVER['HTTP_DEV']))
            $this->is_dev_mode = true;
    }
    //Busca o valor do parâmetro da request, caso o parâmetro não exista aplica o valor default
    public function parametro(&$request, $nome, $default = null){
        try{
            return $request->getParameter($nome);
        }catch(\Exception $e){
            return $default;
        }
    }
    //Busca o valor de um array associativo, caso o parâmetro não exista aplica o valor default
    public function valor($array_assoc, $nome, $default = null){
        if(!$array_assoc || !is_array($array_assoc) || !is_string($nome)) return $default;
        if(!isset($array_assoc[$nome])) return $default;
        return $array_assoc[$nome];
    }
    //Registra uma mensagem para retorno da integração. Caso $erro seja true, troca o status para E (erro)
    public function incluiMensagem($msg, $erro=false){
        if($erro)
            $this->status = 'E';
        if(!empty($msg) && is_string($msg))
            $this->mensagens[] = $msg;
    }
    //Indica o status atual baseado nas mensagens de erro registradas
    public function statusAtual(){
        return $this->status;
    }
    //Indica se as flags de desenvolvimento foram indicadas 
    public function modoDesenvolvedor(){
        return $this->is_dev_mode;
    }
    //Retorna uma resposta da aplicação indicando que é um retorno json (sem estruturas específicas do framework)
    public function retornoJSON(&$response, $data){
        header('Content-Type: application/json; charset=utf-8');
        $response->setMetadata($data);
    }
    //Retorna um json em formato específico STATUS - MESAGEM com o que foi registrado até aqui de mensagens e status
    public function retornoJSONStatusMensagem(&$response){
        $data = [
            'STATUS' => $this->status,
            'MENSAGEM' => $this->mensagens
        ];
        self::retornoJSON($response, $data);
    }
    
    //Verifica se o texto informado é uma data válida para o formato indicado
    public function dataValida($dataEmString, $formato){
        $dateTime = \DateTime::createFromFormat($formato, $dataEmString);
        
        if ($dateTime !== false && !array_sum($dateTime::getLastErrors())) {
            return true;
        } else {
            return false;
        }
    }
}