<?php
namespace Geral\Repositories;

class Retirada extends \Doctrine\ORM\EntityRepository
{

    public function consultaRetiradaPelaRequisicao($nrorg, $filial, $requisicao, $dtRetirada, $servico){
        $sql = "SELECT DISTINCT RETIRADA.NRREQUEXTRRE
                  FROM CARDADIA, RETIRADA
                 WHERE RETIRADA.NRORG = :P_NRORG
                   AND RETIRADA.CDFILIAL = :P_CDFILIAL
                   AND RETIRADA.NRREQUEXTRRE = :P_NRREQUEXTR
                   AND CARDADIA.NRORG = RETIRADA.NRORG
                   AND CARDADIA.CDFILIAL = RETIRADA.CDFILIAL
                   AND CARDADIA.NRCARDAPIO = RETIRADA.NRCARDAPIO
                   AND CARDADIA.DTCARDAPIO = :P_DTRETIRADA
                   AND CARDADIA.CDSERVFILI = :P_CDSERVFILI";

        $param = array(
            'P_NRORG' => $nrorg,
            'P_CDFILIAL' => $filial,
            'P_NRREQUEXTR' => $requisicao,
            'P_DTRETIRADA' => $dtRetirada,
            'P_CDSERVFILI' => $servico,
        );

        $entityManager = $this->getEntityManager();
        $result = $entityManager->getConnection()->fetchColumn($sql, $param);
        return $result ? $result : false;
    }

    public function verificaRetiradaPeloServico($nrorg, $filial, $dtRetirada, $servico){
        /*Verifica se há retirada relizada para o serviço, data e filial selecionados nas tabelas RETIRADA
        e CARDADIA.*/
        try{
          $sql = "SELECT * FROM CARDADIA, RETIRADA
          WHERE RETIRADA.NRORG      = :P_NRORG
          AND INSTR(:P_CDFILIAL, CARDADIA.CDFILIAL) <> 0
          AND CARDADIA.NRORG        = RETIRADA.NRORG
          AND CARDADIA.CDFILIAL     = RETIRADA.CDFILIAL
          AND CARDADIA.NRCARDAPIO   = RETIRADA.NRCARDAPIO
          AND CARDADIA.DTCARDAPIO   = :P_DTRETIRADA
          AND INSTR(:P_CDSERVFILI, CARDADIA.CDSERVFILI) <> 0";

          $param = array(
            'P_NRORG' => $nrorg,
            'P_CDFILIAL' => $filial,
            'P_DTRETIRADA' => $dtRetirada,
            'P_CDSERVFILI' => $servico,
            );

          $entityManager = $this->getEntityManager();
          $result = $entityManager->getConnection()->fetchColumn($sql, $param);

          return $result ? false : true;

        }catch(ExceptionRetiradaGeral $e){
          throw $e;
        }
      }

      public function verificaRegistroRetGer($nrorg, $filial, $dtretgeral, $cdservfili){
        try{
          $sql = 'SELECT RET.*
          FROM RETIRADAGERAL RET
          WHERE RET.NRORG    = :P_NRORG
          AND RET.CDFILIAL   = :P_CDFILIAL
          AND RET.DTRETGERAL = :P_DTRETGERAL
          AND INSTR(:P_CDSERVFILI, RET.CDSERVFILI) <> 0';

          $params = array(
            'P_NRORG'      => $nrorg,
            'P_CDFILIAL'   => $filial,
            'P_DTRETGERAL' => $dtretgeral,
            'P_CDSERVFILI' => $cdservfili,
            );

          $entityManager = $this->getEntityManager();
          $resultSetMapping = new \Doctrine\ORM\Query\ResultSetMappingBuilder($entityManager);
          $resultSetMapping->addRootEntityFromClassMetadata('\Indeterminado\Entities\Retiradageral', "RET");

          $result = $entityManager->createNativeQuery($sql, $resultSetMapping)->execute($params);
          return count($result) > 0 ? $result : false;

        }catch(ExceptionRetiradaGeral $e){
          throw $e;
        }
      }

      public function separaQtConsumidaPorServico($cdfilial, $dtretgeral){
        $sql = "SELECT
                    T.CDFILIAL, NRRETGERAL, CDARVPROD, NMPRODUTO, SGUNIDADE, CDTIPORETI, NMTIPORETI, R.CDPRODUTO,
                    T.TOTALPREVISTA, TOTALCONSUMIDA, TOTALEXTRA, TOTALDEVOLVIDA, QTPREVISRETI, QTEXTRARETI, QTDEVOLVRETI,
                    QTRETIRADA, VRUNILANCTO, CDFILICPLA, DTCONSPLAN, NRCONSPLAN, CDSERVCPLA, NMSERVICO, CDTIPOPRAT,
                    NMTIPOPRAT, CDPRATO, NMPRATO, CDFILAUXPRAT
                FROM
                    ( 
                    SELECT
                    RE.CDFILIAL,
                    RE.NRRETGERAL,
                    PR.CDARVPROD,
                    PR.NMPRODUTO,
                    PR.SGUNIDADE,
                    TR.CDTIPORETI,
                    TR.NMTIPORETI,
                    PR.CDPRODUTO,
                    SUM(RE.QTPRERETIGER) AS QTPREVISRETI,
                    RE.QTEXTRETIGER AS QTEXTRARETI,
                    RE.QTDEVRETIGER AS QTDEVOLVRETI,
                    RE.QTCONRETIGER AS QTRETIRADA,
                    RE.VRUNIRETGER AS VRUNILANCTO,
                    RE.CDFILIAL AS CDFILICPLA,
                    RE.DTRETGERAL AS DTCONSPLAN,
                    '' AS NRCONSPLAN,
                    RE.CDSERVFILI AS CDSERVCPLA,
                    S.NMSERVICO,
                    RE.CDTIPOPRAT,
                    TP.NMTIPOPRAT,
                    RE.CDPRATO,
                    PA.NMPRATO,
                    RE.CDFILAUXPRAT
                      FROM
                        PRODUTO  PR,
                        TIPORETI TR,
                        SERVICO S,
                        RETIRADAGERAL RE,
                        TIPOPRAT TP, 
                        PRATO PA
                      WHERE
                        (RE.CDFILIAL = :P_CDFILIAL)          AND
                        (RE.DTRETGERAL = :P_DTRETGERAL)        AND
                        (PR.CDPRODUTO  = RE.CDPRODUTO)         AND
                        (TR.CDTIPORETI = RE.CDTIPORETI)        AND
                        (S.CDFILIAL(+) = RE.CDFILIAL)          AND
                        (S.CDSERVFILI(+) = RE.CDSERVFILI)      AND
                        (TP.CDTIPOPRAT(+) = RE.CDTIPOPRAT)     AND
                        (PA.CDFILAUXPRAT(+) = RE.CDFILAUXPRAT) AND
                        (PA.CDPRATO(+) = RE.CDPRATO)
                      GROUP BY
                        RE.NRRETGERAL,      
                        PR.CDARVPROD,
                        PR.NMPRODUTO,
                        PR.SGUNIDADE,
                        TR.CDTIPORETI,
                        TR.NMTIPORETI,
                        PR.CDPRODUTO,
                        RE.QTEXTRETIGER,
                        RE.QTDEVRETIGER,
                        RE.QTCONRETIGER,
                        RE.VRUNIRETGER,
                        RE.CDFILIAL,
                        RE.DTRETGERAL,
                        RE.CDSERVFILI,
                        S.NMSERVICO, 
                        RE.CDTIPOPRAT,
                        TP.NMTIPOPRAT,
                        RE.CDPRATO,
                        PA.NMPRATO,
                        RE.CDFILAUXPRAT
                ) R, (SELECT
                        DISTINCT CDPRODUTO, 
                        SUM(QTPRERETIGER) AS TOTALPREVISTA,
                        SUM(QTCONRETIGER) AS TOTALCONSUMIDA,
                        SUM(QTEXTRETIGER) AS TOTALEXTRA,
                        SUM(QTDEVRETIGER) AS TOTALDEVOLVIDA,
                        CDFILIAL
                      FROM RETIRADAGERAL
                      WHERE CDFILIAL = :P_CDFILIAL AND
                        DTRETGERAL = :P_DTRETGERAL AND
        				QTCONRETIGER > 0
                      GROUP BY CDPRODUTO, CDFILIAL) T
                      WHERE T.CDFILIAL = R.CDFILIAL
                        AND T.CDPRODUTO = R.CDPRODUTO
                      ORDER BY
                        CDFILICPLA,
                        CDARVPROD,
                        NMPRODUTO,
                        CDTIPOPRAT,
                        CDPRATO,
                        CDFILAUXPRAT";

        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_DTRETGERAL" => $dtretgeral,
        );

        $entityManager = $this->getEntityManager();
        $result = $entityManager->getConnection()->fetchAll($sql, $params);
        return count($result)>0 ? $result : array();
    }  
    

      public function retornaQtPorServico($cdfilial, $dtretgeral, $servico){
        $sql = "SELECT
                    T.CDFILIAL, NRRETGERAL, CDARVPROD, NMPRODUTO, SGUNIDADE, CDTIPORETI, NMTIPORETI, R.CDPRODUTO,
                    T.TOTALPREVISTA, TOTALCONSUMIDA, TOTALEXTRA, TOTALDEVOLVIDA, QTPREVISRETI, QTEXTRARETI, QTDEVOLVRETI,
                    QTRETIRADA, VRUNILANCTO, CDFILICPLA, DTCONSPLAN, NRCONSPLAN, CDSERVCPLA, NMSERVICO, CDTIPOPRAT,
                    NMTIPOPRAT, CDPRATO, NMPRATO, CDFILAUXPRAT
                FROM
                    ( 
                    SELECT
                    RE.CDFILIAL,
                    RE.NRRETGERAL,
                    PR.CDARVPROD,
                    PR.NMPRODUTO,
                    PR.SGUNIDADE,
                    TR.CDTIPORETI,
                    TR.NMTIPORETI,
                    PR.CDPRODUTO,
                    SUM(RE.QTPRERETIGER) AS QTPREVISRETI,
                    RE.QTEXTRETIGER AS QTEXTRARETI,
                    RE.QTDEVRETIGER AS QTDEVOLVRETI,
                    RE.QTCONRETIGER AS QTRETIRADA,
                    RE.VRUNIRETGER AS VRUNILANCTO,
                    RE.CDFILIAL AS CDFILICPLA,
                    RE.DTRETGERAL AS DTCONSPLAN,
                    '' AS NRCONSPLAN,
                    RE.CDSERVFILI AS CDSERVCPLA,
                    S.NMSERVICO,
                    RE.CDTIPOPRAT,
                    TP.NMTIPOPRAT,
                    RE.CDPRATO,
                    PA.NMPRATO,
                    RE.CDFILAUXPRAT
                      FROM
                        PRODUTO  PR,
                        TIPORETI TR,
                        SERVICO S,
                        RETIRADAGERAL RE,
                        TIPOPRAT TP, 
                        PRATO PA
                      WHERE
                        (RE.CDFILIAL = :P_CDFILIAL)          AND
                        (RE.DTRETGERAL = :P_DTRETGERAL)        AND
        				INSTR(:P_CDSERVFILI, RE.CDSERVFILI) <> 0  AND
                        (PR.CDPRODUTO  = RE.CDPRODUTO)         AND
                        (TR.CDTIPORETI = RE.CDTIPORETI)        AND
                        (S.CDFILIAL(+) = RE.CDFILIAL)          AND
                        (S.CDSERVFILI(+) = RE.CDSERVFILI)      AND
                        (TP.CDTIPOPRAT(+) = RE.CDTIPOPRAT)     AND
                        (PA.CDFILAUXPRAT(+) = RE.CDFILAUXPRAT) AND
                        (PA.CDPRATO(+) = RE.CDPRATO)
                      GROUP BY
                        RE.NRRETGERAL,      
                        PR.CDARVPROD,
                        PR.NMPRODUTO,
                        PR.SGUNIDADE,
                        TR.CDTIPORETI,
                        TR.NMTIPORETI,
                        PR.CDPRODUTO,
                        RE.QTEXTRETIGER,
                        RE.QTDEVRETIGER,
                        RE.QTCONRETIGER,
                        RE.VRUNIRETGER,
                        RE.CDFILIAL,
                        RE.DTRETGERAL,
                        RE.CDSERVFILI,
                        S.NMSERVICO, 
                        RE.CDTIPOPRAT,
                        TP.NMTIPOPRAT,
                        RE.CDPRATO,
                        PA.NMPRATO,
                        RE.CDFILAUXPRAT
                ) R, (SELECT
                        DISTINCT CDPRODUTO, 
                        SUM(QTPRERETIGER) AS TOTALPREVISTA,
                        SUM(QTCONRETIGER) AS TOTALCONSUMIDA,
                        SUM(QTEXTRETIGER) AS TOTALEXTRA,
                        SUM(QTDEVRETIGER) AS TOTALDEVOLVIDA,
                        CDFILIAL
                      FROM RETIRADAGERAL
                      WHERE CDFILIAL = :P_CDFILIAL AND
                        DTRETGERAL = :P_DTRETGERAL AND
        				INSTR(:P_CDSERVFILI, CDSERVFILI) <> 0
                      GROUP BY CDPRODUTO, CDFILIAL) T
                      WHERE T.CDFILIAL = R.CDFILIAL
                        AND T.CDPRODUTO = R.CDPRODUTO
                      ORDER BY
                        CDFILICPLA,
                        CDARVPROD,
                        NMPRODUTO,
                        CDTIPOPRAT,
                        CDPRATO,
                        CDFILAUXPRAT";

        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_DTRETGERAL" => $dtretgeral,
        	"P_CDSERVFILI" => $servico,
        );

        $entityManager = $this->getEntityManager();
        $result = $entityManager->getConnection()->fetchAll($sql, $params);
        return count($result)>0 ? $result : array();
    }  
    

    public function setaValorUnitario($cdfilial, $cdproduto, $dtposicao, $almoxarife, $localEst, $nrLote, $subLote){
        $sql = "SELECT
        P.CDPRODUTO,
        P.NMPRODUTO,
        P.SGUNIDADE,
        IDCNTRESTOQ,
        P.VRFATOCONV,
        TRUNC((D.QTESTOQDIA/P.VRFATOCONV), 5) AS QTESTOQDIA,
        P.CDPRODESTO,
        CUSTO_PRODUTO_LCEST(:P_CDFILIAL, :P_CDPRODUTO, TO_NUMBER(NVL(PF.CDTIPCUSTEST, '3')),
        TO_DATE(:P_DTPOSICAO) - 1, SYSDATE, '$almoxarife','$localEst', '$nrLote',
        '$subLote') VRCUSTPROD
        FROM PRODUTO P, POSESTDIA D, PRODFILI F, PARAMFILIAL PF
        WHERE (P.CDPRODUTO    = :P_CDPRODUTO)    AND
            (D.CDFILIAL     = :P_CDFILIAL)     AND
            (D.CDALMOXARIFE = '$almoxarife') AND
            (D.CDLOCALESTOQ = '$localEst') AND
            (D.NRLOTEESTQ   = '$nrLote')   AND
            (D.NRSUBLOTE    = '$subLote')    AND
            (D.CDPRODUTO    = P.CDPRODESTO)    AND
            (P.CDPRODESTO   = F.CDPRODUTO)     AND
            (F.CDFILIAL     = :P_CDFILIAL)     AND
            (F.CDFILIAL     = PF.CDFILIAL)     AND
            (DTPOSIESTQ = (SELECT MAX(DTPOSIESTQ)
        FROM POSESTDIA
        WHERE (CDFILIAL     = :P_CDFILIAL)     AND
            (CDALMOXARIFE = '$almoxarife') AND
            (CDLOCALESTOQ = '$localEst') AND
            (NRLOTEESTQ   = '$nrLote')   AND
            (NRSUBLOTE    = '$subLote')    AND
            (CDPRODUTO    = P.CDPRODESTO)    AND
            (DTPOSIESTQ  <= :P_DTPOSICAO)))";

        $params = array(
            "P_CDFILIAL"     => $cdfilial,
            "P_CDPRODUTO"    => $cdproduto,
            "P_DTPOSICAO"    => $dtposicao,
        );

        $entityManager = $this->getEntityManager();
        $result = $entityManager->getConnection()->fetchAll($sql, $params);
        return count($result)>0 ? $result : array();

    }

    public function retornaCusto($nrorg, $cdfilial, $cdproduto, $nrtipocusto, $dtreferencia, $dtatual){
      $sql = "SELECT 
              CUSTO_PRODUTO(:P_CDFILIAL, :P_CDPRODUTO, :P_NRTIPOCUSTO, :P_DTREFERENCIA, :P_DTATUAL) CUSTO
              FROM PRODUTO 
              WHERE PRODUTO.CDPRODUTO = :P_CDPRODUTO AND
                PRODUTO.NRORG = :P_NRORG";
      $params = array(
            "P_NRORG"        => $nrorg,
            "P_CDFILIAL"     => $cdfilial,
            "P_CDPRODUTO"    => $cdproduto,
            "P_NRTIPOCUSTO"  => $nrtipocusto,
            "P_DTREFERENCIA" => $dtreferencia,
            "P_DTATUAL"      => $dtatual,
        );

      $entityManager = $this->getEntityManager();
      $result = $entityManager->getConnection()->fetchColumn($sql, $params);
      return count($result)>0 ? $result : false;
    }

    public function retornaQtConsumida($cdfilial, $cdproduto, $dtatual){
      $nrorg = \Geral\Util\Ambiente::getNrorg();
      $sql = "SELECT QTESTOQDIA
              FROM POSESTDIA
              WHERE CDPRODUTO = :P_CDPRODUTO
                AND CDFILIAL = :P_CDFILIAL
                AND DTPOSIESTQ <= :P_DTATUAL
                AND NRORG = :P_NRORG
                ORDER BY DTPOSIESTQ DESC";
      $params = array(
            "P_CDPRODUTO" => $cdproduto,
            "P_CDFILIAL"  => $cdfilial,
            "P_DTATUAL"   => $dtatual,
            "P_NRORG"     => $nrorg,
        );

      $entityManager = $this->getEntityManager();
      $result = $entityManager->getConnection()->fetchAll($sql, $params);
      return $result ? $result : false;
    }

    public function retornaIdcardifreti($cdfilial){
      $nrorg = \Geral\Util\Ambiente::getNrorg();
      $sql = "SELECT IDCARDIFRETI
             FROM 
             PARAMPLANEJA 
             WHERE NRORG = :P_NRORG
             AND CDFILIAL = :P_CDFILIAL";
      $params = array(
            'P_CDFILIAL' => $cdfilial,
            'P_NRORG'    => $nrorg,
        );

      $entityManager = $this->getEntityManager();
      $result = $entityManager->getConnection()->fetchColumn($sql, $params);
      return $result == 'S' ? true : false;
    }

    public function retornaCdTipoRetiradaPadrao($cdfilial){
    $nrorg = \Geral\Util\Ambiente::getNrorg();
    $sql = "SELECT CDTPRETIPADR
             FROM 
             PARAMPLANEJA 
             WHERE NRORG = :P_NRORG
             AND CDFILIAL = :P_CDFILIAL";
      $params = array(
            'P_CDFILIAL' => $cdfilial,
            'P_NRORG'    => $nrorg,
        );

      $entityManager = $this->getEntityManager();
      $result = $entityManager->getConnection()->fetchColumn($sql, $params);
      return $result ? $result : false;
    }
    
    public function retornaIdPermiteVisualizarSaldoEstoque($cdfilial){
        $nrorg = \Geral\Util\Ambiente::getNrorg();
        $sql = "SELECT IDPERMVISPOSEST
    			FROM
    			PARAMFILIAL
    			WHERE NRORG = :P_NRORG
    			AND CDFILIAL = :P_CDFILIAL";
    	$params = array(
    			'P_CDFILIAL' => $cdfilial,
                'P_NRORG'    => $nrorg,
    	);
    	
    	$entityManager = $this->getEntityManager();
    	$result = $entityManager->getConnection()->fetchColumn($sql, $params);
    	return $result == 'S' ? true : false;
    }

    public function retornaQuantidadeConsumida($nrorg, $cdProduto){
        $sql = "SELECT QTPRERETIGER
                FROM RETIRADAGERAL
                WHERE NRORG = :P_NRORG
                AND CDPRODUTO = :P_CDPRODUTO
                AND DTRETGERAL = :P_DTRETGERAL
                GROUP BY QTPRERETIGER";
        $params = array(
                'P_NRORG'     =>$nrorg,
                'P_CDPRODUTO' => $cdProduto,
                'P_DTRETGERAL' => $dtRetirada
        );

        $entityManager = $this->getEntityManager();
        $result = $entityManager->getConnection()->fetchAll($sql, $params);
        return $result ? $result : false;
    }

}