/* Tabela ALIQIMPFIS: TABELA DE ALIQUOTAS DA IMPRESSORA*/
/* Descrio: A trigger insere registros na tabela LOGOPERAVND a partir de alteraes na tabela ALIQIMPFIS */
/* Objetivo: Gerar um log que vai registrar as alteraes na ALIQIMPFIS,
             utilizando a tabela de Logs de Operaes do VND*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'LOG_VND_ALIQIMPFIS' AND
                 TYPE   = 'TR' 	               AND
	              U.UID  = O.UID           AND
	              U.NAME = USER)
DROP TRIGGER LOG_VND_ALIQIMPFIS
GO


CREATE TRIGGER LOG_VND_ALIQIMPFIS
ON ALIQIMPFIS
FOR INSERT, UPDATE, DELETE
AS
  DECLARE @V_DSREMOTEADDR VARCHAR(15);
  DECLARE @V_DSREMOTEHOST VARCHAR(15);
  DECLARE @V_DSSERVERRDM  VARCHAR(20);
  DECLARE @V_DSSERVMETHOD VARCHAR(30);
  DECLARE @V_DSREMOTEOPER VARCHAR(12);
  DECLARE @V_DESC         VARCHAR(500);
  DECLARE @V_ITPESQ       VARCHAR(40);
  DECLARE @V_NMOPERADOR   VARCHAR(40);
  DECLARE @V_IDTIPOOPER   VARCHAR(1);

/*VARIVEIS DO CURSOR C_CURSOR*/

  DECLARE @V_ID_COMANDO    VARCHAR(1);

  DECLARE @V_NEW_CDFILIAL            varchar(4);
  DECLARE @V_NEW_CDPRODUTO           varchar(10);
  DECLARE @V_NEW_VRPEALIMPFIS        double precision;
  DECLARE @V_NEW_CDIMPOSTO           varchar(2);
  DECLARE @V_NEW_IDTPIMPOSFIS        varchar(1);
  DECLARE @V_NEW_DTULTATUALIF        datetime;
  DECLARE @V_NEW_VRALIQIBPT          double precision;
  DECLARE @V_NEW_VRALIQIBPTES        double precision;
  DECLARE @V_NEW_CDCSTICMS           varchar(3);
  DECLARE @V_NEW_CDCSTPISCOF         varchar(2);
  DECLARE @V_NEW_CDCFOPPFIS          varchar(10);
  DECLARE @V_NEW_VRALIQPIS           varchar(10);
  DECLARE @V_NEW_VRALIQCOFINS        varchar(10);
  DECLARE @V_NEW_VRPERCREDUCAO       double precision;
  DECLARE @V_NEW_CDCEST              varchar(9);

  DECLARE @V_OLD_CDFILIAL            varchar(4);
  DECLARE @V_OLD_CDPRODUTO           varchar(10);
  DECLARE @V_OLD_VRPEALIMPFIS        double precision;
  DECLARE @V_OLD_CDIMPOSTO           varchar(2);
  DECLARE @V_OLD_IDTPIMPOSFIS        varchar(1);
  DECLARE @V_OLD_DTULTATUALIF        datetime;
  DECLARE @V_OLD_VRALIQIBPT          double precision;
  DECLARE @V_OLD_VRALIQIBPTES        double precision;
  DECLARE @V_OLD_CDCSTICMS           varchar(3);
  DECLARE @V_OLD_CDCSTPISCOF         varchar(2);
  DECLARE @V_OLD_CDCFOPPFIS          varchar(10);
  DECLARE @V_OLD_VRALIQPIS           varchar(10);
  DECLARE @V_OLD_VRALIQCOFINS        varchar(10);
  DECLARE @V_OLD_VRPERCREDUCAO       double precision;
  DECLARE @V_OLD_CDCEST              varchar(9);

BEGIN
/* IDTIPOOPER - '1' - Incluso, '2' - Alterao, '3' - Excluso*/

   SET @V_DESC   = '';
   SET @V_ITPESQ     = '';
   SET @V_NMOPERADOR = '';
   SET @V_IDTIPOOPER = '';

   EXEC NMUSUARIO @P_NMUSUARIO = @V_NMOPERADOR OUTPUT;
   SET @V_NMOPERADOR = ISNULL(@V_NMOPERADOR, ' ');

   EXEC WEBTUNNING  @P_DSREMOTEADDR = @V_DSREMOTEADDR OUTPUT,
                    @P_DSREMOTEHOST = @V_DSREMOTEHOST OUTPUT,
                    @P_DSSERVERRDM  = @V_DSSERVERRDM OUTPUT,
                    @P_DSSERVMETHOD = @V_DSSERVMETHOD OUTPUT,
                    @P_DSREMOTEOPER = @V_DSREMOTEOPER OUTPUT;

   DECLARE C_CURSOR CURSOR FOR
     SELECT
     'D'   ID_COMANDO,
           NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
	   DELETED.CDFILIAL,
 	   DELETED.CDPRODUTO,
	   DELETED.VRPEALIMPFIS,
	   DELETED.CDIMPOSTO,
	   DELETED.IDTPIMPOSFIS,
	   DELETED.DTULTATUALIF,
	   DELETED.VRALIQIBPT,
	   DELETED.VRALIQIBPTES,
	   DELETED.CDCSTICMS,
	   DELETED.CDCSTPISCOF,
	   DELETED.CDCFOPPFIS,
	   DELETED.VRALIQPIS,
	   DELETED.VRALIQCOFINS,
	   DELETED.VRPERCREDUCAO,
	   DELETED.CDCEST
     FROM
           DELETED
     WHERE     
           DELETED.CDFILIAL + DELETED.CDPRODUTO + DELETED.CDIMPOSTO NOT IN (SELECT  CDFILIAL + CDPRODUTO + CDIMPOSTO   FROM INSERTED)
     UNION ALL

     SELECT
     'U'   IDCOMANDO,
  	   INSERTED.CDFILIAL,
	   INSERTED.CDPRODUTO,
	   INSERTED.VRPEALIMPFIS,
	   INSERTED.CDIMPOSTO,
	   INSERTED.IDTPIMPOSFIS,
 	   INSERTED.DTULTATUALIF,
	   INSERTED.VRALIQIBPT,
	   INSERTED.VRALIQIBPTES,
	   INSERTED.CDCSTICMS,
	   INSERTED.CDCSTPISCOF,
	   INSERTED.CDCFOPPFIS,
	   INSERTED.VRALIQPIS,
	   INSERTED.VRALIQCOFINS,
	   INSERTED.VRPERCREDUCAO,
	   INSERTED.CDCEST,
	   DELETED.CDFILIAL,
 	   DELETED.CDPRODUTO,
	   DELETED.VRPEALIMPFIS,
	   DELETED.CDIMPOSTO,
	   DELETED.IDTPIMPOSFIS,
	   DELETED.DTULTATUALIF,
	   DELETED.VRALIQIBPT,
	   DELETED.VRALIQIBPTES,
	   DELETED.CDCSTICMS,
	   DELETED.CDCSTPISCOF,
	   DELETED.CDCFOPPFIS,
	   DELETED.VRALIQPIS,
	   DELETED.VRALIQCOFINS,
	   DELETED.VRPERCREDUCAO,
	   DELETED.CDCEST
     FROM
           INSERTED,
           DELETED
     WHERE     
  	   INSERTED.CDFILIAL  = DELETED.CDFILIAL  AND  
	   INSERTED.CDPRODUTO = DELETED.CDPRODUTO AND
	   INSERTED.CDIMPOSTO = DELETED.CDIMPOSTO
     UNION ALL

     SELECT
     'I'   IDCOMANDO,
  	   INSERTED.CDFILIAL,
	   INSERTED.CDPRODUTO,
	   INSERTED.VRPEALIMPFIS,
	   INSERTED.CDIMPOSTO,
	   INSERTED.IDTPIMPOSFIS,
 	   INSERTED.DTULTATUALIF,
 	   INSERTED.VRALIQIBPT,
	   INSERTED.VRALIQIBPTES,
	   INSERTED.CDCSTICMS,
	   INSERTED.CDCSTPISCOF,
	   INSERTED.CDCFOPPFIS,
	   INSERTED.VRALIQPIS,
	   INSERTED.VRALIQCOFINS,
	   INSERTED.VRPERCREDUCAO,
	   INSERTED.CDCEST,
           NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
     FROM
           INSERTED
     WHERE     
           INSERTED.CDFILIAL + INSERTED.CDPRODUTO + INSERTED.CDIMPOSTO   NOT IN (SELECT CDFILIAL + CDPRODUTO + CDIMPOSTO FROM DELETED)

     OPEN C_CURSOR;
     FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,
                     @V_NEW_CDFILIAL,@V_NEW_CDPRODUTO,@V_NEW_VRPEALIMPFIS,@V_NEW_CDIMPOSTO,
                     @V_NEW_IDTPIMPOSFIS,@V_NEW_DTULTATUALIF,@V_NEW_VRALIQIBPT,@V_NEW_VRALIQIBPTES,
                     @V_NEW_CDCSTICMS,@V_NEW_CDCSTPISCOF,@V_NEW_CDCFOPPFIS,@V_NEW_VRALIQPIS,
                     @V_NEW_VRALIQCOFINS,@V_NEW_VRPERCREDUCAO,@V_NEW_CDCEST,@V_OLD_CDFILIAL,
                     @V_OLD_CDPRODUTO,@V_OLD_VRPEALIMPFIS,@V_OLD_CDIMPOSTO,@V_OLD_IDTPIMPOSFIS,
                     @V_OLD_DTULTATUALIF,@V_OLD_VRALIQIBPT,@V_OLD_VRALIQIBPTES,@V_OLD_CDCSTICMS,
                     @V_OLD_CDCSTPISCOF,@V_OLD_CDCFOPPFIS,@V_OLD_VRALIQPIS,@V_OLD_VRALIQCOFINS,
                     @V_OLD_VRPERCREDUCAO,@V_OLD_CDCEST

    WHILE (@@FETCH_STATUS = 0) BEGIN
    
       IF @V_ID_COMANDO= 'U' /*UPDATE*/ BEGIN
		IF ISNULL(@V_NEW_CDFILIAL,'NULL') <> ISNULL(@V_OLD_CDFILIAL,'NULL') BEGIN
			SET @V_DESC   = 'Filial ' + ISNULL(@V_NEW_CDFILIAL,'');
		END;
		IF ISNULL(@V_NEW_CDPRODUTO,'NULL') <> ISNULL(@V_OLD_CDPRODUTO,'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + ' Cd Produto ' + ISNULL(@V_NEW_CDPRODUTO,'');
		END;
		IF ISNULL(CONVERT(VARCHAR,@V_NEW_VRPEALIMPFIS),'NULL') <> ISNULL(CONVERT(VARCHAR,@V_OLD_VRPEALIMPFIS),'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + ' Vr do Imposto ' + ISNULL(CONVERT(VARCHAR,@V_NEW_VRPEALIMPFIS),'');
		END;
		IF ISNULL(@V_NEW_CDIMPOSTO,'NULL') <> ISNULL(@V_OLD_CDIMPOSTO,'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + ' Cd Imposto ' + ISNULL(@V_NEW_CDIMPOSTO,'');
		END;
		IF ISNULL(CONVERT(VARCHAR,@V_NEW_DTULTATUALIF),'NULL') <> ISNULL(CONVERT(VARCHAR,@V_OLD_DTULTATUALIF),'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + ' Dt Ult Atu Imposto ' + ISNULL(CONVERT(VARCHAR,@V_NEW_DTULTATUALIF),'');
		END;
		IF ISNULL(CONVERT(VARCHAR,@V_NEW_VRALIQIBPT),'NULL') <> ISNULL(CONVERT(VARCHAR,@V_OLD_VRALIQIBPT),'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + ' IBPT (Federal) ' + ISNULL(CONVERT(VARCHAR,@V_NEW_VRALIQIBPT),'');
		END;
		IF ISNULL(CONVERT(VARCHAR,@V_NEW_VRALIQIBPTES),'NULL') <> ISNULL(CONVERT(VARCHAR,@V_OLD_VRALIQIBPTES),'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + ' IBPT (Estadual) ' + ISNULL(CONVERT(VARCHAR,@V_NEW_VRALIQIBPTES),'');
		END;
		IF ISNULL(@V_NEW_CDCSTICMS,'NULL') <> ISNULL(@V_OLD_CDCSTICMS,'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + ' CST/CSOSN (ICMS) ' + ISNULL(@V_NEW_CDCSTICMS,'');
		END;
		IF ISNULL(@V_NEW_CDCSTPISCOF,'NULL') <> ISNULL(@V_OLD_CDCSTPISCOF,'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + ' CST (PIS/COFINS) ' + ISNULL(@V_NEW_CDCSTPISCOF,'');
		END;
		IF ISNULL(@V_NEW_CDCFOPPFIS,'NULL') <> ISNULL(@V_OLD_CDCFOPPFIS,'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + ' CFOP ' + ISNULL(@V_NEW_CDCFOPPFIS,'');
		END;
		IF ISNULL(@V_NEW_VRALIQPIS,'NULL') <> ISNULL(@V_OLD_VRALIQPIS,'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + ' Alq. PIS ' + ISNULL(@V_NEW_VRALIQPIS,'');
		END;
		IF ISNULL(@V_NEW_VRALIQCOFINS,'NULL') <> ISNULL(@V_OLD_VRALIQCOFINS,'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + ' Alq. COFINS ' + ISNULL(@V_NEW_VRALIQCOFINS,'');
		END;
		IF ISNULL(CONVERT(VARCHAR,@V_NEW_VRPERCREDUCAO),'NULL') <> ISNULL(CONVERT(VARCHAR,@V_OLD_VRPERCREDUCAO),'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + ' Perc. Reduo (ICMS) ' + ISNULL(CONVERT(VARCHAR,@V_NEW_VRPERCREDUCAO),'');
		END;
		IF ISNULL(@V_NEW_CDCEST,'NULL') <> ISNULL(@V_OLD_CDCEST,'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + ' CEST ' + ISNULL(@V_NEW_CDCEST,'');
		END;
		IF ISNULL(@V_NEW_IDTPIMPOSFIS,'NULL') <> ISNULL(@V_OLD_IDTPIMPOSFIS,'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + ' Id Imposto Impresora ' + ISNULL(@V_NEW_IDTPIMPOSFIS,'');
		END;
          SET @V_IDTIPOOPER = '2';
       END;
       IF @V_ID_COMANDO= 'I' /*INSERO*/ BEGIN
          SET @V_DESC   = 'Filial '       + ISNULL(@V_NEW_CDFILIAL ,'') + ' ' +
                  'Cd Produto '           + ISNULL(@V_NEW_CDPRODUTO,'') + ' ' +
                  'Vr do Imposto '        + ISNULL(CONVERT(VARCHAR,@V_NEW_VRPEALIMPFIS),'') + ' ' +
                  'Cd Imposto '           + ISNULL(@V_NEW_CDIMPOSTO,'') + ' ' +
                  'Dt Ult Atu Imposto '   + ISNULL(CONVERT(VARCHAR,@V_NEW_DTULTATUALIF),'') + ' ' +
                  'IBPT (Federal) '       + ISNULL(CONVERT(VARCHAR,@V_NEW_VRALIQIBPT  ),'') + ' ' +
                  'IBPT (Estadual) '      + ISNULL(CONVERT(VARCHAR,@V_NEW_VRALIQIBPTES),'') + ' ' +
                  'CST/CSOSN (ICMS) '     + ISNULL(@V_NEW_CDCSTICMS   ,'') + ' ' +
                  'CST (PIS/COFINS) '     + ISNULL(@V_NEW_CDCSTPISCOF ,'') + ' ' +
                  'CFOP '                 + ISNULL(@V_NEW_CDCFOPPFIS  ,'') + ' ' +
                  'Alq. PIS '            + ISNULL(@V_NEW_VRALIQPIS   ,'') + ' ' +
                  'Alq. COFINS '         + ISNULL(@V_NEW_VRALIQCOFINS,'') + ' ' +
                  'Perc. Reduo (ICMS) ' + ISNULL(CONVERT(VARCHAR,@V_NEW_VRPERCREDUCAO),'') + ' ' +
                  'CEST '                 + ISNULL(@V_NEW_CDCEST      ,'') + ' ' +
                  'Id Imposto Impresora ' + ISNULL(@V_NEW_IDTPIMPOSFIS,'');
          SET @V_IDTIPOOPER = '1';
       END;
       IF @V_ID_COMANDO= 'D' /*DELEO*/  BEGIN
          SET @V_DESC   = 'Filial '       + ISNULL(@V_OLD_CDFILIAL ,'') + ' ' +
                  'Cd Produto '           + ISNULL(@V_OLD_CDPRODUTO,'') + ' ' +
                  'Vr do Imposto '        + ISNULL(CONVERT(VARCHAR,@V_OLD_VRPEALIMPFIS),'') + ' ' +
                  'Cd Imposto '           + ISNULL(@V_OLD_CDIMPOSTO,'') + ' ' +
                  'Dt Ult Atu Imposto '   + ISNULL(CONVERT(VARCHAR,@V_OLD_DTULTATUALIF),'') + ' ' +
                  'IBPT (Federal) '       + ISNULL(CONVERT(VARCHAR,@V_OLD_VRALIQIBPT  ),'') + ' ' +
                  'IBPT (Estadual) '      + ISNULL(CONVERT(VARCHAR,@V_OLD_VRALIQIBPTES),'') + ' ' +
                  'CST/CSOSN (ICMS) '     + ISNULL(@V_OLD_CDCSTICMS   ,'') + ' ' +
                  'CST (PIS/COFINS) '     + ISNULL(@V_OLD_CDCSTPISCOF ,'') + ' ' +
                  'CFOP '                 + ISNULL(@V_OLD_CDCFOPPFIS  ,'') + ' ' +
                  'Alq. PIS '            + ISNULL(@V_OLD_VRALIQPIS   ,'') + ' ' +
                  'Alq. COFINS '         + ISNULL(@V_OLD_VRALIQCOFINS,'') + ' ' +
                  'Perc. Reduo (ICMS) ' + ISNULL(CONVERT(VARCHAR,@V_OLD_VRPERCREDUCAO),'') + ' ' +
                  'CEST '                 + ISNULL(@V_OLD_CDCEST      ,'') + ' ' +
                  'Id Imposto Impresora ' + ISNULL(@V_OLD_IDTPIMPOSFIS,'');
          SET @V_IDTIPOOPER = '3';
       END;
       IF  (@V_IDTIPOOPER <> '') BEGIN
          IF @V_DESC IS NOT NULL BEGIN
             SET @V_ITPESQ =  ISNULL(@V_NEW_CDFILIAL+@V_NEW_CDPRODUTO+@V_NEW_CDIMPOSTO , @V_OLD_CDFILIAL+@V_OLD_CDPRODUTO+@V_OLD_CDIMPOSTO);
             SET @V_DESC = SUBSTRING(@V_DESC, 1, 300);
             INSERT INTO LOGOPERAVND
                    (NMTABELAVND, DTOPERACAOVND, NMRESPOPERVND, IDTIPOOPERVND, DSOPERACAOVND,
                     CDPESQITEMVND, DSRDMUSUOPERVND, DSENDUSUOPERVND, DSNMUSUOPERVND, DSMETUSUOPERVND,
                     CDORESPOPERVND, CDFILLOGOPERVND)
             VALUES ('ALIQIMPFIS',GETDATE(), ISNULL(@V_NMOPERADOR,' '), @V_IDTIPOOPER, @V_DESC, @V_ITPESQ,
                     @V_DSSERVERRDM, @V_DSREMOTEADDR, @V_DSREMOTEHOST, @V_DSSERVMETHOD, @V_DSREMOTEOPER,
                     ISNULL(@V_NEW_CDFILIAL,@V_OLD_CDFILIAL));
          END;
       END;
       FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,
                       @V_NEW_CDFILIAL,@V_NEW_CDPRODUTO,@V_NEW_VRPEALIMPFIS,@V_NEW_CDIMPOSTO,
                       @V_NEW_IDTPIMPOSFIS,@V_NEW_DTULTATUALIF,@V_NEW_VRALIQIBPT,@V_NEW_VRALIQIBPTES,
                       @V_NEW_CDCSTICMS,@V_NEW_CDCSTPISCOF,@V_NEW_CDCFOPPFIS,@V_NEW_VRALIQPIS,
                       @V_NEW_VRALIQCOFINS,@V_NEW_VRPERCREDUCAO,@V_NEW_CDCEST,@V_OLD_CDFILIAL,
                       @V_OLD_CDPRODUTO,@V_OLD_VRPEALIMPFIS,@V_OLD_CDIMPOSTO,@V_OLD_IDTPIMPOSFIS,
                       @V_OLD_DTULTATUALIF,@V_OLD_VRALIQIBPT,@V_OLD_VRALIQIBPTES,@V_OLD_CDCSTICMS,
                       @V_OLD_CDCSTPISCOF,@V_OLD_CDCFOPPFIS,@V_OLD_VRALIQPIS,@V_OLD_VRALIQCOFINS,
                       @V_OLD_VRPERCREDUCAO,@V_OLD_CDCEST

    SET @V_DESC   = '';
    END;
    CLOSE      C_CURSOR;
    DEALLOCATE C_CURSOR;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'LOG_VND_ALIQIMPFIS','5.20.001';
GO 